package com.zh.project0512.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.mapper.QywxDepartmentUserLinkMapper;
import com.zh.project0512.mapper.RankMapper;
import com.zh.project0512.mapper.UserMapper;
import com.zh.project0512.model.*;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.model.vo.RankListVo;
import com.zh.project0512.service.IRewardRuleService;
import com.zh.project0512.service.ITaskService;
import com.zh.project0512.service.IUserPointsRecordsService;
import com.zh.project0512.service.IVideoService;
import com.zh.project0512.utils.AppMessageUtil;
import com.zh.project0512.utils.HttpUtil;
import com.zh.project0512.utils.PropertyUtils;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Tag(name = "定时任务")
@RestController
public class ScheduleController {
    @Autowired
    private ITaskService taskService;
    @Resource
    QywxDepartmentUserLinkMapper qywxDepartmentUserLinkMapper;
    @Resource
    RankMapper rankMapper;
    @Autowired
    private IVideoService videoService;
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private IRewardRuleService rewardRuleService;
    @Autowired
    private IUserPointsRecordsService userPointsRecordsService;
    @Autowired
    private HttpUtil httpUtil;
    @Scheduled(cron = "0 0 0 * * ?")
    public void taskBegin() {
        UpdateWrapper<Task> ew = new UpdateWrapper<>();
        LocalDateTime now = LocalDateTime.now();
        ew.eq("status", 0).eq("date_format(start, '%Y-%m-%d')", now.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")))
            .set("status", 1).set("updateAt", now);
        taskService.update(ew);
    }

    @Scheduled(cron = "0 0 0 * * ?")
    @Transactional
    public void taskEnd() {
        UpdateWrapper<Task> ew = new UpdateWrapper<>();
        LocalDateTime now = LocalDateTime.now();
        String format = now.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        ew.eq("status", 1).eq("date_format(end, '%Y-%m-%d')", format)
            .set("status", 2).set("updateAt", LocalDateTime.now());
        taskService.update(ew);
        videoService.updStatusWhenTaskEnd(format);
    }

    // 按任务结算积分
    @Scheduled(cron = "0 0 0 * * ?")
    @Transactional
    public void videoEffect() {
        // 任务结束7天时，自动结算积分
        QueryWrapper<Task> qw = new QueryWrapper<>();
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime weekAgo = now.minusWeeks(1);
        AppMessage appMessage = new AppMessage().setTitle("任务积分结算");
        String format = weekAgo.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        qw.eq("status",2).eq("date_format(end, '%Y-%m-%d')", format);
        List<Task> taskList = taskService.list(qw);
        String qywxAccessToken = httpUtil.qywxGetToken();
        // 处理每个任务的传播值
        for (Task t : taskList) {
            Integer taskId = t.getId();
            List<Map> videoEffectResult = videoService.videoEffectResult(taskId);
            List<Map> rewardRule = rewardRuleService.listByTemplateId(t.getRewardRuleTemplateId());
            List<UserPointsRecords> recordsList = new ArrayList<>();
            System.out.println("videoEffectResult" + videoEffectResult);
            // 处理任务中每个用户的传播值
            for (Map m : videoEffectResult) {
                BigDecimal bigDecimal = (BigDecimal) m.get("effectResultSum");
                Integer effectResultSum = Integer.parseInt(bigDecimal.toString());
                if (effectResultSum != null && effectResultSum != 0) {
                    int point = 0;
                    int userId = (Integer) m.get("userId");
                    String touser = (String) m.get("touser");
                    // 根据传播值计算规则算出奖励
                    for (Map r : rewardRule) {
                        if (effectResultSum > (Integer) r.get("limitNum")) {
                            point = (Integer) r.get("reward");
                        }
                    }
                    // （视频结算标识）并更新用户积分,发送企业消息和小程序内消息
                    userMapper.updPoints(userId, point);
                    appMessage.setDescription("恭喜您获得星途积分" + point + "。")
                        .setUrl("pages/quest/subpage/detail/detail?id=" + taskId)
                        .setReceiverId(userId);
                    // 企业微信消息通知
                    if (touser != null) {
                        appMessage.setTouser(touser);
                        List<JSONObject> content_item = new ArrayList<>();
                        content_item.add(new JSONObject().fluentPut("key", "到账结果").fluentPut("value", appMessage.getDescription()));
                        content_item.add(new JSONObject().fluentPut("key", "到账时间").fluentPut("value", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"))));
                        httpUtil.qywxMessage(qywxAccessToken, touser, "任务积分结算", null, appMessage.getUrl(), content_item);
                    }
                    AppMessageUtil.sendMessage(appMessage);
                    recordsList.add(new UserPointsRecords().setSettlementMethod(1).setPoints(point).setType(1).setRemarks("任务完成结算："+t.getTitle()).setCreateDate(now).setUserId(userId));
                }
            }
            // 按任务生成积分流水记录
            userPointsRecordsService.saveBatch(recordsList);
            taskService.update(new UpdateWrapper<Task>().eq("id",taskId).set("status",3));
        }
    }
//    @Scheduled(cron = "0 0/1 * * * ? ")
//    @Scheduled(cron = "0 0 0 1 1/1 ?")
//    public void taskRank() {
//        //获取开始时间和结束时间
//        Calendar cal = Calendar.getInstance();
//        cal.setTime(new Date());
//        cal.add(Calendar.MONTH,-1);
//        cal.set(Calendar.DAY_OF_MONTH,1);
//        cal.set(Calendar.HOUR_OF_DAY, 0);
//        cal.set(Calendar.MINUTE, 0);
//        cal.set(Calendar.SECOND, 0);
//        Date timeStart = cal.getTime();
//        cal.set(Calendar.DAY_OF_MONTH,cal.getActualMaximum(Calendar.DAY_OF_MONTH));
//        Date timeEnd = cal.getTime();
//        //获取员工排名进排名数据库
//        QueryWrapper<RankListDTO> queryWrapper = new QueryWrapper<>();
//        queryWrapper.groupBy("u.id");
//        queryWrapper.orderByDesc("value");
//        List<RankListVo> rankUserList = qywxDepartmentUserLinkMapper.getUserRankList(queryWrapper);
//        if (rankUserList != null && rankUserList.size()>0){
//            for (RankListVo rankListVo : rankUserList) {
//                //查询是否有上一月记录，如果有就减去上一月传播值，没有则直接添加
//                QueryWrapper<Rank> queryWrapper2 = new QueryWrapper<>();
//                queryWrapper2.eq("linkId",rankListVo.getLinkId());
//                queryWrapper2.eq("type",2);//2.员工
//                queryWrapper2.orderByDesc("dateStart");
//                queryWrapper2.last("limit 1");
//                Rank rank = rankMapper.selectOne(queryWrapper2);
//                if (rank != null){
//                    rankListVo.setValue(rankListVo.getValue() - rank.getValue());
//                }
//                Rank rank2 = new Rank();
//                PropertyUtils.copyProperties(rankListVo,rank2);
//                rank2.setType(2);//2.员工
//                rank2.setDateStart(timeStart);
//                rank2.setDateEnd(timeEnd);
//                rank2.setCreateDate(new Date());
//                rankMapper.insert(rank2);
//            }
//        }
//        //计算经销商排名进排名数据库
//        QueryWrapper<RankListDTO> queryWrapper3 = new QueryWrapper<>();
//        queryWrapper3.groupBy("qd.departmentId");
//        queryWrapper3.orderByDesc("value");
//        List<RankListVo> rankDepartmentList = qywxDepartmentUserLinkMapper.getDepartmentRankList(queryWrapper3);
//        if (rankDepartmentList != null && rankDepartmentList.size()>0){
//            for (RankListVo rankListVo : rankDepartmentList) {
//                //查询是否有上一月记录，如果有就减去上一月传播值，没有则直接添加
//                QueryWrapper<Rank> queryWrapper2 = new QueryWrapper<>();
//                queryWrapper2.eq("linkId",rankListVo.getLinkId());
//                queryWrapper2.eq("type",1);//1.经销商
//                queryWrapper2.orderByDesc("dateStart");
//                queryWrapper2.last("limit 1");
//                Rank rank = rankMapper.selectOne(queryWrapper2);
//                if (rank != null){
//                    rankListVo.setValue(rankListVo.getValue() - rank.getValue());
//                }
//                Rank rank2 = new Rank();
//                PropertyUtils.copyProperties(rankListVo,rank2);
//                rank2.setType(1);//1.经销商
//                rank2.setDateStart(timeStart);
//                rank2.setDateEnd(timeEnd);
//                rank2.setCreateDate(new Date());
//                rankMapper.insert(rank2);
//            }
//        }
//    }
}
