package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.Tag;
import com.zh.project0512.service.ITagService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;


/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-22
 */

@RestController
@RequestMapping("/manage/tag")
@io.swagger.v3.oas.annotations.tags.Tag(name = "标签")
public class TagController {
    @Autowired
    private ITagService tagService;

    @Data
    static class AddTagParam {
        @NotNull(message = "title不能为空")
        @Schema(title = "标签名称")
        private String title;
    }

    @Operation(summary = "创建标签")
    @PostMapping("/add")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "120")
    public Result add(@Validated @RequestBody AddTagParam param) {
        tagService.addTag(new Tag().setTitle(param.getTitle()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "标签id")
        private Integer id;
    }

    @Operation(summary = "删除标签(软删除)")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "121")
    public Result del(@Validated @RequestBody DelTagParam param) {
        return MybatisPlusUtil.sqlResult(tagService.update(new UpdateWrapper<Tag>().eq("id", param.getId()).set("sortWeight",0).set("isDeleted", 1)), "删除");
    }

    @Data
    static class UpdTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "标签id")
        private Integer id;
        @NotNull(message = "name不能为空")
        @Schema(title = "标签名称")
        private String title;
    }

    @Operation(summary = "更新标签名称")
    @PostMapping("/upd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "131")
    public Result upd(@Validated @RequestBody UpdTagParam param) {
        return MybatisPlusUtil.sqlResult(tagService.updateById(new Tag().setId(param.getId()).setTitle(param.getTitle()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Operation(summary = "标签列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@RequestBody(required = false) JSONObject obj) {
        QueryWrapper<Tag> qw = new QueryWrapper<>();
        qw.eq("isDeleted",0).orderByAsc("sortWeight").orderByDesc("updateAt", "creatAt");
        return Result.success(tagService.pageMaps(MybatisPlusUtil.SetPage(obj), qw));
    }

    @Data
    static class UpdBSParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "标签id")
        private Integer id;
        @NotNull(message = "排序权重不能为空")
        @Min(value = 1, message = "排序权重最小值为1")
        @Schema(title = "排序权重:数字越小排序靠前")
        private Integer sortWeight;
    }

    @Operation(summary = "修改标签排序")
    @PostMapping("/updSort")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "132")
    public Result updSort(@Validated @RequestBody UpdBSParam param) {
        Tag tag = tagService.getById(param.getId());
        if(tag.getIsDeleted() == 1){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"该标签已被删除");
        }
        int origin = tag.getSortWeight();
        int current = param.getSortWeight();
        if (origin == current) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "权重无变化");
        }
        tagService.updSort(origin, current, origin > current);
        return Result.success("修改完成");
    }
}
