package com.zh.project0512.mapper;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.model.User;
import com.zh.project0512.model.Video;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
public interface VideoMapper extends BaseMapper<Video> {
    // 任务有效时间到达后视频进入审核状态
    @Update("update video INNER JOIN task on task.id = video.taskId and date_format(task.end, '%Y-%m-%d') = #{now}\n" +
        " set video.status = 1 \n" +
        "WHERE video.status = 0")
    void updStatusWhenTaskEnd(String now);

    // 任务手动结束后视频进入审核状态
    @Update("update video INNER JOIN task on task.id = video.taskId and taskId = #{taskId} set video.status = 1 WHERE video.status = 0 ")
    void updStatusByTaskId(int taskId, LocalDateTime endTime);

    @Select("SELECT t1.*,t2.name as providerName,t3.name as departmentName,t4.title as taskTitle,t5.account,t5.name as accountName,t5.tel as accountTel,t5.validImg as accountImg \n" +
        "  from video as t1\n" +
        "LEFT JOIN user as t2 on t2.id = t1.userId\n" +
        "LEFT JOIN qywxDepartment as t3 on t3.departmentId = t2.main_department\n" +
        "LEFT JOIN task as t4 on t4.id = t1.taskId\n" +
        "LEFT JOIN userAccount as t5 on t1.userId = t5.userId and t1.type = t5.platform\n" +
        "${ew.customSqlSegment} " +
        "ORDER BY updateAt DESC,creatAt DESC")
    IPage<Map> pageList(IPage iPage, @Param("ew") Wrapper<Video> queryWrapper);

    @Select("SELECT sum(t1.playNum) as playNumSum,\n" +
        "sum(t1.commendNum) as commendNumSum ,\n" +
        "sum(t1.collectionNum) as collectionNumSum ,\n" +
        "sum(t1.commentNum) as commentNumSum ,\n" +
        "sum(t1.reSendNum) as reSendNumSum ,\n" +
        "sum(t1.recommendNum) as recommendNumSum \n" +
        " from video as t1" +
        " inner join user on user.openid = #{openid} and user.id = t1.userId\n" +
        "where t1.status = 2")
    JSONObject record(String openid);
    // 任务下用户的视频数据统计
    @Select("SELECT type\n" +
        ",IFNULL(SUM(playNum),0) as playNumSum\n" +
        ",IFNULL(SUM(commendNum),0) as commendNumSum\n" +
        ",IFNULL(SUM(collectionNum),0) as collectionNumSum\n" +
        ",IFNULL(SUM(commentNum),0) as commentNumSum\n" +
        ",IFNULL(SUM(reSendNum),0) as reSendNumSum\n" +
        ",IFNULL(SUM(recommendNum),0) as recommendNumSum\n" +
        " from video as t1\n" +
        " INNER JOIN user as t2 on t2.id =t1.userId\n" +
        "${ew.customSqlSegment}")
    List<Map> videoDataByUserTask(@Param("ew") Wrapper<Video> queryWrapper);
    // 任务下视频影响值统计
    @Select("SELECT t1.id,t1.taskId,t2.title as taskTitle,t1.userId,t3.name as userName,t3.userId as touser,SUM(IFNULL(t1.effectResult,0)) as effectResultSum \n" +
        "from video as t1\n" +
        "INNER JOIN task as t2 on t1.taskId = t2.id\n" +
        "INNER JOIN user as t3 on t1.userId = t3.id\n" +
        "WHERE taskId = #{taskId} and t1.status = 2\n" +
        "GROUP BY t1.userId \n" +
        "ORDER BY effectResultSum DESC")
    List<Map> videoEffectResult(int taskId);
}
