package com.zh.project0512.model;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author zh
 * @since 2022-05-23
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@Schema
public class Reference extends Model {

    private static final long serialVersionUID = 1L;

    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 名称
     */
    @Schema(title = "名称")
    private String title;

    /**
     * 素材分组id(0为未分组)
     */
    @Schema(title = "素材分组id", description = "0为未分组")
    @TableField("groupId")
    private Integer groupId;

    @Schema(title = "素材分组名称")
    @TableField(exist = false)
    private String groupName;

    /**
     * 类型：1视频；2图片
     */
    @Schema(title = "类型", description = "1视频；2图片")
    private Integer type;

    /**
     * 文件地址
     */
    @Schema(title = "文件地址")
    @NotNull(message = "fileUrl不能为空")
    @TableField("fileUrl")
    private String fileUrl;

    /**
     * 文件缩略图
     */
    @Schema(title = "文件缩略图（图片超高清时方便预加载）")
    @TableField("fileUrlSmall")
    private String fileUrlSmall;

    /**
     * 时长(视频需要)
     */
    @Schema(title = "时长(视频需要)")
    @TableField("duration")
    private Integer duration;

    /**
     * 封面图片地址
     */
    @Schema(title = "封面图片地址", description = "视频文件请上传封面图片")
    @TableField("coverUrl")
    private String coverUrl;

    @TableField("creatAt")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime creatAt;

    @TableField("updateAt")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateAt;

    @Schema(title = "标签id列表")
    @TableField(exist = false)
    private List tagList;

    @Schema(title = "品牌id列表")
    @TableField(exist = false)
    private List brandList;

    @Schema(title = "是否被用户收藏")
    @TableField(exist = false)
    private Integer isCollected;

    /**
     * 是否删除:0否；1是
     */
    @Schema(title = "是否删除:0否；1是")
    @TableField("isDeleted")
    private Integer isDeleted;

}
